#include <windows.h>
#include <math.h>
#include <string.h>
#include "usersurf.h"

/*
Written by Kenneth E. Moore
June 19, 1997
*/

int __declspec(dllexport) APIENTRY UserDefinedSurface(USER_DATA *UD, FIXED_DATA *FD);

/* a generic Snells law refraction routine */
int Refract(double thisn, double nextn, double *l, double *m, double *n, double ln, double mn, double nn);

BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
	{
   return TRUE;
   }

/* this DLL models a cylindrical surface with grating lines equally spaced along the arc of the surface. */

int __declspec(dllexport) APIENTRY UserDefinedSurface(USER_DATA *UD, FIXED_DATA *FD)
	{
	int i;
   double p2, alpha;
   double xpower, ypower, a, b, c, rad, t;
   double T, M, C, L, Y, YC, dpdx, dpdy, nn, nx, ny, nz, ux, uy, uz;

   switch(FD->type)
   	{
      case 0:
      	/* ZEMAX is requesting general information about the surface */
         switch(FD->numb)
         	{
            case 0:
            	/* ZEMAX wants to know the name of the surface */
		         /* do not exceed 12 characters */
		         strcpy(UD->string,"Diff Grt Cyl");
               break;
            case 1:
            	/* ZEMAX wants to know if this surface is rotationally symmetric */
               /* it is not, return a null string */
            	UD->string[0] = '\0';
               break;
            case 2:
            	/* ZEMAX wants to know if this surface is a gradient index media */
               /* it is not. */
            	UD->string[0] = '\0';
            	break;
            }
         break;
      case 1:
      	/* ZEMAX is requesting the names of the parameter columns */
         /* the value FD->numb will indicate which value ZEMAX wants. */
         /* returning a null string indicates that the parameter is unused. */
         switch(FD->numb)
         	{
            case 1:
            	/* Grating spacing */
            	strcpy(UD->string, "Lines/micron");
               break;
            case 2:
            	/* Diffraction order */
            	strcpy(UD->string, "Order");
               break;
            default:
            	UD->string[0] = '\0';
            	break;
            }
      	break;
      case 2:
      	/* ZEMAX is requesting the names of the extra data columns */
         /* the value FD->numb will indicate which value ZEMAX wants. */
         /* they are all "Unused" for this surface type */
         /* returning a null string indicates that the extradata value is unused. */
         switch(FD->numb)
         	{
            default:
            	UD->string[0] = '\0';
            	break;
            }
      	break;
      case 3:
      	/* ZEMAX wants to know the sag of the surface */
         /* if there is an alternate sag, return it as well */
         /* otherwise, set the alternate sag identical to the sag */
         /* The sag is sag1, alternate is sag2. */

         UD->sag1 = 0.0;
         UD->sag2 = 0.0;

			/* if a plane, just return */
			if (FD->cv == 0) return(0);

			/* okay, not a plane */
         /* the surface shape is a cylinder in the YZ plane */
         p2 = UD->y * UD->y;
         alpha = 1 - FD->cv*FD->cv*p2;
         if (alpha < 0) return(-1);
         UD->sag1 = (FD->cv*p2)/(1 + sqrt(alpha));
         UD->sag2 = UD->sag1;
      	break;
      case 4:
      	/* ZEMAX wants a paraxial ray trace to this surface */
         /* x, y, z, and the optical path are unaffected, at least for this surface type */
         /* for paraxial ray tracing, the return z coordinate should always be zero. */
         /* paraxial surfaces are always planes with the following normals */

         UD->ln =  0.0;
         UD->mn =  0.0;
         UD->nn = -1.0;
         xpower = 0.0;
         ypower = (FD->n2 - FD->n1)*FD->cv;
         if ((UD->n) != 0.0)
         	{
            (UD->l) = (UD->l)/(UD->n);
            (UD->m) = (UD->m)/(UD->n);

            (UD->l) = (FD->n1*(UD->l) - (UD->x)*xpower)/(FD->n2);
            (UD->m) = (FD->n1*(UD->m) - (UD->y)*ypower)/(FD->n2);

            /* normalize */
            (UD->n) = sqrt(1/(1 + (UD->l)*(UD->l) + (UD->m)*(UD->m) ) );
            /* de-paraxialize */
            (UD->l) = (UD->l)*(UD->n);
            (UD->m) = (UD->m)*(UD->n);
            }
         break;
      case 5:
      	/* ZEMAX wants a real ray trace to this surface */
         if (FD->cv == 0.0)
         	{
	         UD->ln =  0.0;
   	      UD->mn =  0.0;
      	   UD->nn = -1.0;
			   if (Refract(FD->n1, FD->n2, &UD->l, &UD->m, &UD->n, UD->ln, UD->mn, UD->nn)) return(-FD->surf);
            }
         else
         	{
	         /* okay, not a plane */
				a = (UD->n) * (UD->n) + (UD->m) * (UD->m);
				b = ((UD->n)/FD->cv) - (UD->y) * (UD->m);
				c = (UD->y) * (UD->y);

				rad = b * b - a * c;
				if (rad < 0) return(FD->surf);  /* ray missed this surface */
				if (FD->cv > 0) t = c / (b + sqrt(rad));
				else           t = c / (b - sqrt(rad));
				(UD->x) = (UD->l) * t + (UD->x);
				(UD->y) = (UD->m) * t + (UD->y);
				(UD->z) = (UD->n) * t + (UD->z);
				UD->path = t;

				/* okay, now compute the local normal */
				UD->ln = 0;
				UD->mn = (UD->y) * (FD->cv);
				UD->nn = ((UD->z) - 1.0/(FD->cv)) * (FD->cv);
            }

         if (Refract(FD->n1, FD->n2, &UD->l, &UD->m, &UD->n, UD->ln, UD->mn, UD->nn)) return(-FD->surf);

         /* okay, now account for the grating */
         /* the grating affects both the refraction angle and the phase, or OPD, of the ray */
         T = FD->param[1];
         M = FD->param[2];
         L = FD->wavelength;
         C = FD->cv;
         Y = UD->y;
         YC = Y*C;

			/* here is the phase & phase change */
         /* the ZEMAX convention is we subtract out the positive phase...*/
         dpdx = 0.0; /* included here only for extension to the general case */
         if (C == 0.0 || fabs(YC) >= 1.0)
         	{
            /* on a plane */
	         UD->path += M*T*L*Y / FD->n1;
      	   dpdy = M*T*L;
            }
         else
         	{
	         UD->path += (M*T*L*asin(YC)/C) / FD->n1;
      	   dpdy = M*T*L/sqrt(1.0 - YC*YC);
            }

         /* this is a generic vector diffraction routine more or less from Welford */
         /* if you do not understand how it works, see Welford, don't call me! *.

         /* account for possible change in index or mirrors */
			dpdy /= fabs(FD->n2);
         if (FD->n1*FD->n2 < 0) dpdy = -dpdy;
			if (FD->n1 < 0) nn = -1.0;
			else nn = 1.0;

         nx = -UD->ln;
			ny = -UD->mn;
			nz = -UD->nn;

			ux = UD->l + nn * (dpdx);
			uy = UD->m + nn * (dpdy);
			uz = UD->n;

			rad = nx*ux + ny*uy + nz*uz;
			rad = 1.0 - (ux*ux + uy*uy + uz*uz) + rad*rad;
			if (rad <= 0.0) rad = 0.0;
			else rad = sqrt(rad);
			UD->l = ux - (nx*ux + ny*uy + nz*uz)*nx + nx * rad;
			UD->m = uy - (nx*ux + ny*uy + nz*uz)*ny + ny * rad;
			UD->n = uz - (nx*ux + ny*uy + nz*uz)*nz + nz * rad;

         break;
      case 6:
      	/* ZEMAX wants the index, dn/dx, dn/dy, and dn/dz at the given x, y, z. */
         /* This is only required for gradient index surfaces, so return dummy values */
         UD->index = FD->n2;
         UD->dndx = 0.0;
         UD->dndy = 0.0;
         UD->dndz = 0.0;
         break;
      case 7:
      	/* ZEMAX wants the "safe" data. */
         /* this is used by ZEMAX to set the initial values for all parameters and extra data */
         /* when the user first changes to this surface type. */
         /* this is the only time the DLL should modify the data in the FIXED_DATA FD structure */
			FD->param[1] = 0.0; /* lines/micron */
			FD->param[2] = 1.0; /* order */
         for (i = 3; i <=   8; i++) FD->param[i] = 0.0;
         for (i = 1; i <= 200; i++) FD->xdata[i] = 0.0;
         break;
      }
   return 0;
   }

int Refract(double thisn, double nextn, double *l, double *m, double *n, double ln, double mn, double nn)
{
double nr, cosi, cosi2, rad, cosr, gamma;
if (thisn != nextn)
	{
	nr = thisn / nextn;
	cosi = fabs((*l) * ln + (*m) * mn + (*n) * nn);
	cosi2 = cosi * cosi;
	if (cosi2 > 1) cosi2 = 1;
	rad = 1 - ((1 - cosi2) * (nr * nr));
	if (rad < 0) return(-1);
	cosr = sqrt(rad);
	gamma = nr * cosi - cosr;
	(*l) = (nr * (*l)) + (gamma * ln);
	(*m) = (nr * (*m)) + (gamma * mn);
	(*n) = (nr * (*n)) + (gamma * nn);
	}
return 0;
}

